import 'package:flutter/material.dart';

/// tipos de animaciones
enum AnimationType { normal, fadeIn }

///Main Class [context] es el BuildContext de la aplicacion en ese momento
///[child] es el widget a navegar, [animation] es el tipo de animacion
class RouteTransitions {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  RouteTransitions(
      {required this.context,
      required this.child,
      this.animation = AnimationType.normal,
      this.duration = const Duration(seconds: 1),
      this.replacement = false}) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        this._fadeInTransition();
        break;
    }
  }

  ///Push normal de la pagina
  void _pushPage(Route route) => Navigator.push(context, route);

  ///Push replacement de la pagina
  void _pushReplacePage(Route route) =>
      Navigator.pushReplacement(context, route);

  ///Codigo de una transicion normal
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child);

    (this.replacement) ? this._pushReplacePage(route) : this._pushPage(route);
  }

  ///Controlador de la transicion con FadeIn
  void _fadeInTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) {
          return FadeTransition(
            child: child,
            opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
                CurvedAnimation(parent: animation, curve: Curves.easeInOut)),
          );
        });

    (this.replacement) ? this._pushReplacePage(route) : this._pushPage(route);
  }
}
